local stt = CreateFrame("Frame")

local STTConfigDefaults = {
	enabled = true,
	switch = {
		flagged = false,
		bg = true,
		arena = true,
		world = true,
	},
	chat = true,
}

local function SwitchTab(toPlayer)
    -- this function is for actually changing the tab-target key binding
	-- if toPlayer is true then we're setting tab to only target players
	local bindingString
	if toPlayer then
		bindingString = "TARGETNEARESTENEMYPLAYER"
	else
		bindingString = "TARGETNEARESTENEMY"
	end

	if GetBindingKey(bindingString) then
		-- looping through to make sure the binding isn't already correct
		for i = 1, #{GetBindingKey(bindingString)} do
			if ({GetBindingKey(bindingString)})[i] == "TAB" then
				return
			end
		end
	end

	if (type(coro) == "thread" and coroutine.status(coro) ~= "dead") then
		return
	end

	coro = coroutine.create(function()
		while true do
			-- purpose of this loop/coroutine is to avoid trying to set the keybinding
			-- while we're in combat.
			if not InCombatLockdown() then
				break
			end
			stt:RegisterEvent("PLAYER_REGEN_ENABLED")
			coroutine.yield()
		end

		stt:UnregisterEvent("PLAYER_REGEN_ENABLED")

		SetBinding("TAB", bindingString)
        if STTConfig.chat then
            print("Tab-targetting switched to "..((toPlayer and "players only") or "any enemy"))
        end

	end)

	coroutine.resume(coro)
end

local function StandaloneSwitchTab()
    --switches the binding based on current condition

    if STTConfig.enabled then

        if STTConfig.switch.flagged then
            SwitchTab(UnitIsPVP("player"))
            return

        elseif STTConfig.switch.bg or STTConfig.switch.arena or STTConfig.switch.world then
            if STTConfig.switch.bg or STTConfig.switch.arena then
                for i = 1, MAX_BATTLEFIELD_QUEUES do
                    local status, _, _, _, _, teamSize = GetBattlefieldStatus(i)
                    if status == "active" then
                        if teamSize > 0 and STTConfig.switch.arena then
                            SwitchTab(true)
                            return
                        elseif teamSize == 0 and STTConfig.switch.bg then
                            SwitchTab(true)
                            return
                        end
                    end
                end
            end
            if STTConfig.switch.world then
                SwitchTab(CanHearthAndResurrectFromArea())
                return
            end
        end
    end
    SwitchTab()
end


local function STT_RegisterEvents()
    stt:UnregisterAllEvents()
    stt:RegisterEvent("PLAYER_ENTERING_WORLD")
    -- this function registers events based on when user has configured to
    -- switch tab target binding

    if not STTConfig.enabled then
        return
    end

    if STTConfig.switch.flagged then
        stt:RegisterEvent("PLAYER_FLAGS_CHANGED")
        -- switching tab-target when flagged for pvp obsoletes all other
        -- configuration options
        return
    end

    if STTConfig.switch.world then
        stt:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    end

    -- event for bg/arena switching is "PLAYER_ENTERING_WORLD" which should
    -- always be registered

end


local function STT_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" then
        STTConfig = STTConfig or STTConfigDefaults
        STT_RegisterEvents()
		stt:UnregisterEvent("ADDON_LOADED")

    elseif event == "PLAYER_REGEN_ENABLED" then
		coroutine.resume(coro)

    elseif event == "PLAYER_FLAGS_CHANGED" then
        SwitchTab(UnitIsPVP("player"))

    elseif event == "PLAYER_ENTERING_WORLD" then
        StandaloneSwitchTab()

    elseif event == "ZONE_CHANGED_NEW_AREA" then
		StandaloneSwitchTab()

    end

end

stt:SetScript("OnEvent", STT_OnEvent)
stt:RegisterEvent("ADDON_LOADED")
stt:RegisterEvent("PLAYER_ENTERING_WORLD")


-- options panel

local panel = CreateFrame("Frame")
panel.name = "SmartTabTarget"
panel:Hide()
panel:SetAllPoints()

local fs_title = panel:CreateFontString()
fs_title:SetFontObject("GameFontNormalLarge")
fs_title:SetText("SmartTabTarget")
fs_title:SetPoint("TOPLEFT", 15, -15)

local cb_enabled = CreateFrame("CheckButton", "sttenabled", panel, "OptionsCheckButtonTemplate")
cb_enabled:SetPoint("TOPLEFT", fs_title, "BOTTOMLEFT", 0, -15)
cb_enabled:SetScript("OnUpdate", function() this:SetChecked(STTConfig.enabled) end)
cb_enabled:SetScript("OnClick", function() STTConfig.enabled = not STTConfig.enabled; StandaloneSwitchTab(); STT_RegisterEvents() end)
cb_enabled:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("Enabled") end)

local cb_chat = CreateFrame("CheckButton", "sttchat", panel, "OptionsCheckButtonTemplate")
cb_chat:SetPoint("TOPLEFT", cb_enabled, "BOTTOMLEFT", 0, -10)
cb_chat:SetScript("OnUpdate", function() this:SetChecked(STTConfig.chat); if STTConfig.enabled then this:Enable() else this:Disable() end end)
cb_chat:SetScript("OnClick", function() STTConfig.chat = not STTConfig.chat end)
cb_chat:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("Alert in chat window when binding is switched") end)

local fs_switchwhen = panel:CreateFontString()
fs_switchwhen:SetFontObject("GameFontNormal")
fs_switchwhen:SetText("Switch when...")
fs_switchwhen:SetPoint("TOPLEFT", cb_chat, "BOTTOMLEFT", 0, -15)

local cb_flagged = CreateFrame("CheckButton", "sttflagged", panel, "OptionsCheckButtonTemplate")
cb_flagged:SetPoint("TOPLEFT", fs_switchwhen, "BOTTOMLEFT", 5, -5)
cb_flagged:SetScript("OnUpdate", function() this:SetChecked(STTConfig.switch.flagged); if STTConfig.enabled then this:Enable() else this:Disable() end end)
cb_flagged:SetScript("OnClick", function() STTConfig.switch.flagged = not STTConfig.switch.flagged; StandaloneSwitchTab(); STT_RegisterEvents() end)
cb_flagged:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("PvP Flagged") end)

local cb_bg = CreateFrame("CheckButton", "sttbg", panel, "OptionsCheckButtonTemplate")
cb_bg:SetPoint("TOPLEFT", cb_flagged, "BOTTOMLEFT", 0, -5)
cb_bg:SetScript("OnUpdate", function() this:SetChecked(STTConfig.switch.bg); if STTConfig.enabled and not STTConfig.switch.flagged then this:Enable() else this:Disable() end end)
cb_bg:SetScript("OnClick", function() STTConfig.switch.bg = not STTConfig.switch.bg; StandaloneSwitchTab(); STT_RegisterEvents() end)
cb_bg:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("In a Battleground") end)

local cb_arena = CreateFrame("CheckButton", "sttarena", panel, "OptionsCheckButtonTemplate")
cb_arena:SetPoint("TOPLEFT", cb_bg, "BOTTOMLEFT", 0, -5)
cb_arena:SetScript("OnUpdate", function() this:SetChecked(STTConfig.switch.arena); if STTConfig.enabled and not STTConfig.switch.flagged then this:Enable() else this:Disable() end end)
cb_arena:SetScript("OnClick", function() STTConfig.switch.arena = not STTConfig.switch.arena; StandaloneSwitchTab(); STT_RegisterEvents() end)
cb_arena:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("In an Arena Match") end)

local cb_world = CreateFrame("CheckButton", "sttworld", panel, "OptionsCheckButtonTemplate")
cb_world:SetPoint("TOPLEFT", cb_arena, "BOTTOMLEFT", 0, -5)
cb_world:SetScript("OnUpdate", function() this:SetChecked(STTConfig.switch.world); if STTConfig.enabled and not STTConfig.switch.flagged then this:Enable() else this:Disable() end end)
cb_world:SetScript("OnClick", function() STTConfig.switch.world = not STTConfig.switch.world; StandaloneSwitchTab(); STT_RegisterEvents() end)
cb_world:SetScript("OnShow", function() getglobal(this:GetName().."Text"):SetText("In a World PvP Zone (such as Wintergrasp)") end)

InterfaceOptions_AddCategory(panel)

SLASH_SMARTTABTARGET1, SLASH_SMARTTABTARGET2 = "/smarttabtarget", "/stt"
function SlashCmdList.SMARTTABTARGET()
    InterfaceOptionsFrame_OpenToCategory(panel)
end